@@ -5,6 +5,7 @@ import logging
 from datetime import datetime
 from copy import deepcopy
 import numpy as np
+import pandas as pd
 
 # redis with asyncronous implementation
 from redis import StrictRedis
@@ -18,56 +19,77 @@ import async_timeout
 import gym
 from gym.spaces import Box
 
-from dragg.redis_client import RedisClient
+# from dragg.redis_client import RedisClient
+import dragg.redis_client as rc
 from dragg.logger import Logger
 from dragg.mpc_calc import MPCCalc
 from dragg_comp.agent import RandomAgent
 
+import selectors
+
+
+
 REDIS_URL = "redis://localhost"
 
 class PlayerHome(gym.Env):
-    def __init__(self, redis_url=REDIS_URL):
+    def __init__(self, redis_url=REDIS_URL, home_dict=None):
+        """
+        PlayerHome should be initialized with a REDIS_URL. Initialization with home_dict
+        as an explicit input should be reserved for unit testing of player submissions.
+        """
+        self.log = Logger("player")
         self.nstep = 0
         self.redis_url = redis_url
         asyncio.run(self.await_status("ready"))
-        home = self.set_home()
-        self.home = MPCCalc(home)
+        if redis_url:
+            home_dict = self.get_home_redis()
+        self.home = MPCCalc(home_dict)
         self.name = self.home.name
         with open('data/state_action.json','r') as file:
             states_actions = json.load(file)
         self.states = [k for k, v in states_actions['states'].items() if v]
         self.observation_space = Box(-1, 1, shape=(len(self.states), ))
         self.actions = [k for k, v in states_actions['actions'].items() if v]
-        a_min = []
-        a_max = []
-        for action in self.actions:
-            if action == "hvac_setpoint":
-                a_min += [16]
-                a_max += [24]
-            elif action == "wh_setpoint":
-                a_min += [42]
-                a_max += [52]
-            elif action == "ev_charge":
-                a_min += [-1]
-                a_max += [1]
-        self.action_space = Box(np.array(a_min), np.array(a_max))
-        asyncio.run(self.post_status("initialized player"))
-        asyncio.run(self.await_status("all ready"))
+        self.action_space = Box(-1*np.ones(len(self.actions)), np.ones(len(self.actions)))
+        # print("INITIALIZING PLAYER")
+        # asyncio.run(self.post_status("initialized player"))
+        # print("ALL READY")
+        # loop = asyncio.new_event_loop()
+        # # asyncio.set_event_loop(asyncio.new_event_loop())
+        # asyncio.run(self.await_status("all ready"))
+        selector = selectors.SelectSelector()
+        loop = asyncio.SelectorEventLoop(selector)
+        asyncio.set_event_loop(loop)
+        loop = asyncio.get_event_loop()
+        loop.run_until_complete(self.post_status("initialized player"))
+        loop.run_until_complete(self.await_status("all ready"))
+        loop.close()
+
+
         self.demand_profile = []
         self.reset(initialize=True)
 
+    def update_states(self, obs_dict):
+        self.states = [k for k, v in obs_dict.items() if v]
+        self.observation_space = Box(-1, 1, shape=(len(self.states), ))
+
     def reset(self, initialize=False):
         """
         Reset as required by OpenAI gym. Beta implementation simply returns current observation, 
         meaning that the simulation will overall continue running. 
         :return: state vector of length n
         """
+        print("resetting")
+        self.log.logger.info("Resetting the player's environment.")
+        asyncio.run(self.post_status("reset"))
+        self.nstep = 0
+
         if initialize:
             self.home.redis_get_initial_values()
             self.home.cast_redis_timestep()
             self.home.get_initial_conditions()
-            self.home.add_type_constraints()
-            self.home.set_type_p_grid()
+            self.home.add_base_constraints()
+            self.home.set_p_grid()
             self.home.solve_mpc(debug=True)
             self.home.cleanup_and_finish()
 
@@ -75,27 +97,26 @@ class PlayerHome(gym.Env):
 
         return obs 
 
-    def set_home(self):
+    def get_home_redis(self):
         """
         Gets the first home in the queue (broadcast by the Aggregator).
         :return: MPCCalc object
         :input: None
         """
-        redis_client = RedisClient()
-        self.num_timesteps = int(redis_client.conn.hgetall("simulation")['nsteps'])
-        home = redis_client.conn.hgetall("home_values")
-        home['hvac'] = redis_client.conn.hgetall("hvac_values")
-        home['wh'] = redis_client.conn.hgetall("wh_values")
-        home['hems'] = redis_client.conn.hgetall("hems_values")
+        redis_client = rc.connection(self.redis_url)#RedisClient()
+        self.num_timesteps = int(redis_client.hgetall("simulation")['nsteps'])
+        home = redis_client.hgetall("home_values")
+        home['hvac'] = redis_client.hgetall("hvac_values")
+        home['wh'] = redis_client.hgetall("wh_values")
+        home['hems'] = redis_client.hgetall("hems_values")
         home['hems']["weekday_occ_schedule"] = [[19,8],[17,18]]
         if 'battery' in home['type']:
-            home['battery'] = redis_client.conn.hgetall("battery_values")
+            home['battery'] = redis_client.hgetall("battery_values")
         if 'pv' in home['type']:
-            home['pv'] = redis_client.conn.hgetall("pv_values")
-        home['wh']['draw_sizes'] = [float(i) for i in redis_client.conn.lrange('draw_sizes', 0, -1)]
-        home['hems']['weekday_occ_schedule'] = redis_client.conn.lrange('weekday_occ_schedule', 0, -1)
-        print(f"Welcome {home['name']}")
-
+            home['pv'] = redis_client.hgetall("pv_values")
+        home['wh']['draw_sizes'] = [float(i) for i in redis_client.lrange('draw_sizes', 0, -1)]
+        home['hems']['weekday_occ_schedule'] = redis_client.lrange('weekday_occ_schedule', 0, -1)
+        self.log.logger.info(f"Welcome {home['name']}")
         return home
 
     def get_obs(self):
@@ -107,40 +128,51 @@ class PlayerHome(gym.Env):
         obs = []
         self.obs_dict = {}
         for state in self.states:
+            skip = False
             if state in self.home.optimal_vals.keys():
                 obs += [self.home.optimal_vals[state]]
-                self.obs_dict.update({state:self.home.optimal_vals[state]})
             elif state == "leaving_horizon":
-                obs += [self.home.index_8am[0] if self.home.index_8am else -1]
-                self.obs_dict.update({state:self.home.index_8am[0] if self.home.index_8am else -1})
+                obs += [self.home.optimal_vals[state]]
             elif state == "returning_horizon":
-                obs += [self.home.index_5pm[0] if self.home.index_5pm else -1]
-                self.obs_dict.update({state:self.home.index_5pm[0] if self.home.index_5pm else -1})
+                obs += [self.home.ev.index_5pm[0] if self.home.ev.index_5pm else -1]
             elif state == "occupancy_status":
-                obs += [int(self.home.occ_slice[0])]
-                self.obs_dict.update({state:int(self.home.occ_slice[0])})
+                obs += [int(self.home.occ_on[0])]
             elif state == "future_waterdraws":
-                obs += [self.home.draw_frac.value]
-                self.obs_dict.update({state:self.home.draw_frac.value})
-            elif state == "oat_future":
-                obs += [self.home.oat_current[-1]]
-                self.obs_dict.update({state:self.home.oat_current[-1]})
-            elif state == "oat_current":
-                obs += [self.home.oat_current[0]]
-                self.obs_dict.update({state:self.home.oat_current[-1]})
+                obs += [np.sum(self.home.stored_optimal_vals["waterdraws"]) / self.home.wh.wh_size]
+            elif state == "t_out":
+                obs += [(self.home.all_oat[self.home.start_slice] - np.min(self.home.all_oat)) / (np.max(self.home.all_oat) - np.min(self.home.all_oat))]
+            elif state == "t_out_6hr":
+                obs += [(self.home.all_oat[self.home.start_slice + 6*self.home.dt] - np.min(self.home.all_oat)) / (np.max(self.home.all_oat) - np.min(self.home.all_oat))]
+            elif state == "t_out_12hr":
+                obs += [(self.home.all_oat[self.home.start_slice + 12*self.home.dt] - np.min(self.home.all_oat)) / (np.max(self.home.all_oat) - np.min(self.home.all_oat))]
+            elif state == "ghi":
+                obs += [self.home.all_ghi[self.home.start_slice] / np.max(self.home.all_ghi)]
+            elif state == "ghi_6hr":
+                obs += [self.home.all_ghi[self.home.start_slice + 6*self.home.dt] / np.max(self.home.all_ghi)]
+            elif state == "ghi_12hr":
+                obs += [self.home.all_ghi[self.home.start_slice + 12*self.home.dt] / np.max(self.home.all_ghi)]
+            elif state == "t_in":
+                obs += [(self.home.optimal_vals["temp_in_opt"] - self.home.hvac.t_in_min_current.value[0]) / (self.home.hvac.t_in_max_current.value[0] - self.home.hvac.t_in_min_current.value[0])]
+            elif state == "t_wh":
+                obs += [(self.home.optimal_vals["temp_wh_opt"] - self.home.wh.temp_wh_min.value )/(self.home.wh.temp_wh_max.value - self.home.wh.temp_wh_min.value)]
+            elif state == "e_ev":
+                obs += [self.home.optimal_vals["e_ev_opt"]]
             elif state == "time_of_day":
-                tod = self.home.timestep % (24 * self.home.dt)
+                tod = self.home.timestep % (24 * self.home.dt) / 24
                 obs += [tod]
-                self.obs_dict.update({state:tod})
             elif state == "community_demand":
-                community_demand = self.home.redis_client.conn.hget("current_values", "current_demand")
-                obs += [community_demand]
-                self.obs_dict.update({state:community_demand})
+                community_demand = self.home.redis_client.hget("current_values", "current_demand")
+                if not community_demand:
+                    community_demand = 0
+                obs += [community_demand / (self.home.max_load / 5) - 1]
             elif state == "my_demand":
-                obs += [self.home.stored_optimal_vals["p_grid_opt"][0]]
-                self.obs_dict.update({state:self.home.stored_optimal_vals["p_grid_opt"][0]})
+                obs += [2 * self.home.stored_optimal_vals["p_grid_opt"][0] / self.home.max_load - 1]
             else:
-                print(f"MISSING {state}")
+                skip = True
+                self.log.logger.warn(f"MISSING {state}")
+
+            if not skip:
+                self.obs_dict.update({state:obs[-1]})
 
         return obs
 
@@ -157,10 +189,10 @@ class PlayerHome(gym.Env):
         Calculates a score for the player in the game.
         :return: dictionary of key performance indexes
         """
-        kpis = {"std_demand": np.std(self.demand_profile), "max_demand": np.max(self.demand_profile)}
+        kpis = {"std_demand": [np.std(self.demand_profile)], "max_demand": [np.max(self.demand_profile)]}
 
-        with open("score.txt", 'w'):
-            kpis.write(json.dumps(kpis))
+        kpis_df = pd.DataFrame(kpis)
+        kpis_df.to_csv("outputs/score.csv")
 
         return kpis
 
@@ -178,7 +210,7 @@ class PlayerHome(gym.Env):
 
         self.home.log = pathos.logger(level=logging.INFO, handler=fh, name=self.name)
 
-        self.redis_client = RedisClient()
+        self.redis_client = rc.connection(self.redis_url)#RedisClient()
         self.home.redis_get_initial_values()
         self.home.cast_redis_timestep()
 
@@ -187,22 +219,23 @@ class PlayerHome(gym.Env):
 
         self.home.get_initial_conditions()
 
-        self.home.add_type_constraints()
+        self.home.add_base_constraints()
         if action is not None:
             if "ev_charge" in self.actions:
-                self.home.override_ev_charge(action[-1]) # overrides the p_ch for the electric vehicle
+                self.home.ev.override_charge(action[-1]) # overrides the p_ch for the electric vehicle
                 action = action[:-1]
             if "wh_setpoint" in self.actions:
-                self.home.override_t_wh(action[-1]) # same but for waterheater
+                # wh_action = self.wh_min + action[-1] * (self.wh_max - self.wh_min)
+                self.home.wh.override_p_wh(action[-1]) # same but for waterheater
                 action = action[:-1]
             if "hvac_setpoint" in self.actions:
-                self.home.override_t_in(action[-1]) # changes thermal deadband to new lower/upper bound
+                # hvac_action = self.hvac_min + action[-1] * (self.hvac_max - self.hvac_min)
+                self.home.hvac.override_t_in(action[-1]) # changes thermal deadband to new lower/upper bound
         
-        self.home.set_type_p_grid()
+        self.home.set_p_grid()
         self.home.solve_mpc(debug=True)
         self.home.cleanup_and_finish()
         self.home.redis_write_optimal_vals()
-        # self.home.run_home()
 
         self.home.log.removeHandler(fh)
 
@@ -229,7 +262,7 @@ class PlayerHome(gym.Env):
                 async with async_timeout.timeout(1):
                     message = await pubsub.get_message(ignore_subscribe_messages=True)
                     if message is not None:
-                        print(f"(Reader) Message Received: {message}")
+                        self.log.logger.debug(f"(Reader) Message Received: {message['data'].decode()}")
                         if status in message["data"].decode():
                             break
                         else:
@@ -247,7 +280,7 @@ class PlayerHome(gym.Env):
         async_redis = aioredis.from_url(self.redis_url)
         pubsub = async_redis.pubsub()
         await pubsub.subscribe("channel:1")
-        print(f"{self.home.name} {status} at t = {self.nstep}.")
+        self.log.logger.info(f"{self.home.name} {status} at t = {self.nstep}.")
         await async_redis.publish("channel:1", f"{self.home.name} {status} at t = {self.nstep}.")
         return 
 
@@ -256,8 +289,8 @@ if __name__=="__main__":
     tic = datetime.now()
     my_home = PlayerHome()
 
-    for _ in range(my_home.num_timesteps * my_home.home.dt):
-        action = [random.uniform(16,22), random.uniform(0,1)]
+    for _ in range(my_home.num_timesteps):
+        action = my_home.action_space.sample()
         my_home.step(action) 
 
     asyncio.run(my_home.post_status("done"))
