@@ -23,7 +23,8 @@ from pathos.pools import ProcessPool
 # Local
 from dragg.mpc_calc import MPCCalc, manage_home
 from dragg.aggregator import Aggregator
-from dragg.redis_client import RedisClient
+# from dragg.redis_client import RedisClient
+import dragg.redis_client as rc
 from dragg.logger import Logger
 
 REDIS_URL = "redis://localhost"
@@ -31,7 +32,8 @@ REDIS_URL = "redis://localhost"
 class RLAggregator(Aggregator):
     def __init__(self, start=None, end=None, redis_url=REDIS_URL):
         self.redis_url = redis_url
-        super().__init__(start, end)
+        super().__init__(start, end, redis_url)
+        self.overwrite_output = True
         self.mpc_players = [] # RLAggregator distinguishes between comp controlled (mpc_players) and human players
 
     def get_homes(self):
@@ -59,27 +61,24 @@ class RLAggregator(Aggregator):
             if len(self.all_homes_copy) > 0:
                 next_home = self.all_homes_copy.pop()
             else:
-                print("WARNING: You have initialized more players than are set in the community")
-                # next_home = self.all_homes_copy[0]
+                self.log.logger.warn("You have initialized more players than are set in the community")
 
-            self.redis_client.conn.hset("simulation", "nsteps", self.num_timesteps)
+            self.redis_client.hset("simulation", "nsteps", self.num_timesteps)
             for k, v in next_home.items():
                 if not k in ["wh","hvac","battery","pv","hems"]:
-                    self.redis_client.conn.hset("home_values", k, v)
+                    self.redis_client.hset("home_values", k, v)
                 else:
                     for k2, v2 in v.items():
                         if not k2 in ["draw_sizes", "weekday_occ_schedule"]:
-                            self.redis_client.conn.hset(f"{k}_values", k2, v2)
+                            self.redis_client.hset(f"{k}_values", k2, v2)
                         else:
-                            self.redis_client.conn.delete(k2)
-                            self.redis_client.conn.rpush(k2, *v2)
+                            self.redis_client.delete(k2)
+                            self.redis_client.rpush(k2, *v2)
 
         else:
-            print('initializing mpc players')
             for next_home in self.all_homes_copy:
                 self.mpc_players += [MPCCalc(next_home)]
-                print(f"Aggregator initialized {self.mpc_players[-1].name}")
-            
+                self.log.logger.info(f"Aggregator initialized MPC player {self.mpc_players[-1].name}")
 
         return 
 
@@ -102,11 +101,9 @@ class RLAggregator(Aggregator):
                 async with async_timeout.timeout(1):
                     message = await channel.get_message(ignore_subscribe_messages=True)
                     if message is not None:
-                        # print(f"(Reader) Message Received: {message}")
                         if "initialized player" in message["data"].decode():
-                            print("INITIALIZED PLAYER")
+                            self.log.logger.debug("Initialized a new player")
                             i += 1 
-                            # pretty sure there's an issue here with the time it takes for a time out/sleep
                             if i < self.config['community']['n_players']:
                                 self.post_next_home()
                                 i += 1
@@ -129,7 +126,7 @@ class RLAggregator(Aggregator):
         :return None:
         """
 
-        i = 0
+        i = 1
         self.next_ts = 1
         while True:
             try:
@@ -138,8 +135,7 @@ class RLAggregator(Aggregator):
                     if message is not None:
                         if str(self.next_ts) in message["data"].decode():
                             if "updated" in message["data"].decode():
-                                print(f"(Reader) rl house {i} updated")
-                                i += 1
+                                self.log.logger.info(f"Player {i}/{self.config['community']['n_players']} updated demand at t={self.next_ts}.")
                                 if i == self.config['community']['n_players']: # now we know that the whole community has stepped
                                     self.redis_set_current_values()
                                     self.run_iteration()
@@ -148,14 +144,19 @@ class RLAggregator(Aggregator):
 
                                     i = 0
                                     self.next_ts += 1
-                                    print("(Reader) timestep can be moved forward")
+                                i += 1
 
                         elif "done" in message["data"].decode():
                             self.write_outputs()
                             return # fix this break
 
+                        elif "reset" in message["data"].decode():
+                            self.timestep = 0
+                            self.next_ts = 1
+
                     await asyncio.sleep(0.1)
             except asyncio.TimeoutError:
+                self.log.logger.error("TIMEOUT (No update from MPC players.)")
                 pass
 
         return
@@ -185,8 +186,9 @@ class RLAggregator(Aggregator):
         self.post_next_home()
         self.reset_collected_data()
         
-        print("starting aioredis listener")
-        redis = aioredis.from_url("redis://localhost")
+        # print("starting aioredis listener")
+        self.log.logger.info("Starting aioredis listener...")
+        redis = aioredis.from_url(self.redis_url)
         pubsub = redis.pubsub()
         await pubsub.subscribe("channel:1", "channel:2")
         await redis.publish("channel:1", "ready")
